#include "komplex.h"

#include <iostream>
#include <iomanip>
#include <limits>

#include <cmath>

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif


double Komplex::get_r() const {
    return sqrt(re * re + im * im);
}

double Komplex::get_fi() const {
    return atan2(im, re); // Imre, haha
}

Komplex operator+(Komplex const& c) {
    return c;
}

Komplex operator-(Komplex const& c) {
    return Komplex(-c.get_re(), -c.get_im());
}

Komplex operator+(Komplex const& lhs, Komplex const &rhs) {
    return Komplex(lhs.get_re() + rhs.get_re(), lhs.get_im() + rhs.get_im());
}

Komplex operator-(Komplex const& lhs, Komplex const &rhs) {
    return Komplex(lhs.get_re() - rhs.get_re(), lhs.get_im() - rhs.get_im());
}

Komplex operator*(Komplex const& lhs, Komplex const &rhs) {
    return Komplex_from_r_fi(lhs.get_r() * rhs.get_r(), lhs.get_fi() + rhs.get_fi());
//    return Komplex(
//            get_re() * rhs.get_re() - get_im() * rhs.get_im(),
//            get_re() * rhs.get_im() + get_im() * rhs.get_re());
}

Komplex operator/(Komplex const& lhs, Komplex const &rhs) {
    return Komplex_from_r_fi(lhs.get_r() / rhs.get_r(), lhs.get_fi() - rhs.get_fi());
}

Komplex& operator+=(Komplex& lhs, Komplex const &rhs) {
    lhs = lhs + rhs;
    return lhs;
}

Komplex& operator-=(Komplex& lhs, Komplex const &rhs) {
    lhs = lhs - rhs;
    return lhs;
}

Komplex& operator*=(Komplex& lhs, Komplex const &rhs) {
    lhs = lhs * rhs;
    return lhs;
}

Komplex& operator/=(Komplex& lhs, Komplex const &rhs) {
    lhs = lhs / rhs;
    return lhs;
}

bool operator==(Komplex const& lhs, Komplex const& rhs) {
    double e = 1e-10;
    return std::fabs(lhs.get_re() - rhs.get_re()) < e &&
           std::fabs(lhs.get_im() - rhs.get_im()) < e;
}

bool operator!=(Komplex const& lhs, Komplex const& rhs) {
    return !(lhs == rhs);
}

std::ostream& operator<<(std::ostream& os, Komplex const& k) {
    os << k.get_re();
    if (k.get_im() >= 0)
        os << "+";
    os << k.get_im() << "i";
    return os;
}

Komplex Komplex_from_r_fi(double r, double fi) {
    return Komplex(r * cos(fi), r * sin(fi));
}

Komplex operator+(double d, Komplex const& k) {
    return k + d;
}

Komplex operator-(double d, Komplex const& k) {
    return -k + d;
}

Komplex Komplex_sqrt(double d) {
    if (d >= 0)
        return Komplex(sqrt(d));
    else
        return Komplex(0, sqrt(-d));
}

Komplex operator/(Komplex const &k, double d) {
    return Komplex(k.get_re() / d, k.get_im() / d);
}
